Attribute VB_Name = "modComparePaths"
Option Explicit
' ---------------------------------------------------------------------
'   Notice:
'   Source code provided by LA Solutions Ltd as-is without warranty of
'   fitness for purpose.  You may use this code for whatever purpose
'   private or commercial provided that this notice is retained in full
'   www.la-solutions.co.uk
'   End of notice
' ---------------------------------------------------------------------
'   This project references the
'   Microsoft Windows Script Host Object Model (IWshRuntimeLibrary)
'   delivered in WINDOWS\system32\wshom.ocx, and
'   the Microsoft Windows Scripting Runtime (Scripting) delivered in scrrun.dll
' ---------------------------------------------------------------------
Sub TestCompareUncWithDrivePath()
    Const drivePath                         As String = "N:\Word\"
    Const uncPath                           As String = "\\Asterix\OfficeDir\Word\"
    
    If (CompareUncWithDrivePath(uncPath, drivePath)) Then
        MsgBox "Paths match", vbInformation Or vbOKOnly, "LA Solutions"
    Else
        MsgBox "Paths don't match", vbCritical Or vbOKOnly, "LA Solutions"
    End If
End Sub
' ---------------------------------------------------------------------
'   ParseDriveLetter
'   Extracts the drive letter from a path
' ---------------------------------------------------------------------
Public Function ParseDriveLetter(ByVal path As String) As String
    ParseDriveLetter = vbNullString
    On Error GoTo err_ParseDriveLetter
    Dim oFileSystem                         As New Scripting.FileSystemObject
    
    Dim oFolder                             As Scripting.Folder
    Set oFolder = oFileSystem.GetFolder(path)
    If (oFolder Is Nothing) Then
        Debug.Print "ParseDriveLetter: Folder '" & path & "' is invalid"
    Else
        ParseDriveLetter = oFileSystem.GetDriveName(oFolder.path)
        Debug.Print "ParseDriveLetter: " & oFileSystem.GetDriveName(oFolder.path)
    End If
    Set oFolder = Nothing
    Set oFileSystem = Nothing
    Exit Function
    
err_ParseDriveLetter:
    Select Case Err.number
    Case 76:
        '   Path not found -- invalid drive letter or letter not mapped
        Debug.Print "Path '" & path & "' not found"
    Case Else
        MsgBox "Error no. " & CStr(Err.number) & ": " & Err.Description & vbNewLine & _
            "Was caused by " & Err.Source, vbOKOnly Or vbExclamation, "Error in function ParseDriveLetter"
    End Select
End Function
' ---------------------------------------------------------------------
'   GetMappedPathFromDrive
'   Attempt to convert a drive letter to a UNC path
'   Returns:    Boolean True if successfull
'   Example:
'   Input       M:
'   Output      \\Asterix\OfficeDir
' ---------------------------------------------------------------------
Public Function GetMappedPathFromDrive(ByRef unc As String, ByVal drive As String) As Boolean
    GetMappedPathFromDrive = False
    On Error GoTo err_GetMappedPathFromDrive
    
    Dim oWshNetwork                         As New WshNetwork
    Dim oDrives                             As New WshCollection
    '   The EnumNetworkDrives method returns a collection.
    '   This collection is an array that associates pairs of items  network drive local names and their associated UNC names.
    '   Even-numbered items in the collection represent local names of logical drives.
    '   Odd-numbered items represent the associated UNC share names.
    '   The first item in the collection is at index zero (0)
    Set oDrives = oWshNetwork.EnumNetworkDrives
    Debug.Print "GetMappedPathFromDrive: " & CStr(oDrives.Count / 2) & " mapped drives"
    Dim i                                   As Integer
    For i = 0 To oDrives.Count - 1 Step 2
        '   Drive is oDrives.Item(i), UNC is oDrives.Item(i + 1)
        Debug.Print "Drive '" & oDrives.Item(i) & "' mapped UNC '" & oDrives.Item(i + 1) & "'"
        If (0 = StrComp(drive, oDrives.Item(i), vbTextCompare)) Then
            unc = oDrives.Item(i + 1)
            GetMappedPathFromDrive = True
            Exit For
        End If
    Next
    Set oDrives = Nothing
    Set oWshNetwork = Nothing
    Exit Function
    
err_GetMappedPathFromDrive:
    MsgBox "Error no. " & CStr(Err.number) & ": " & Err.Description & vbNewLine & _
        "Was caused by " & Err.Source, vbOKOnly Or vbExclamation, "Error in function GetMappedPathFromDrive"
End Function
' ---------------------------------------------------------------------
'   CompareUncWithDrivePath
'   Compares a drive path with a UNC path.
'   Returns: Boolean True if paths map to the same location
'   Example:
'   UNC    \\server\common\cad\dgn\
'   drive  D:\cad\dgn\
'   If these map to the same location, function returns True
' ---------------------------------------------------------------------
Public Function CompareUncWithDrivePath(ByVal uncPath As String, ByVal drivePath As String) As Boolean
    CompareUncWithDrivePath = False
    On Error GoTo err_CompareUncWithDrivePath
    Debug.Print "CompareUncWithDrivePath: DrivePath '" & drivePath & "'; UNC '" & uncPath & "'"
    Dim driveLetter                         As String
    Dim unc                                 As String
    driveLetter = ParseDriveLetter(drivePath)
    If (GetMappedPathFromDrive(unc, driveLetter)) Then
        Debug.Print "Matched " & drivePath
        Dim path                            As String
        path = Replace(drivePath, driveLetter, unc)
        Debug.Print "Substituted path '" & path & "'"
        If (0 = StrComp(path, uncPath)) Then
            CompareUncWithDrivePath = True
        End If
    Else
        Debug.Print drivePath & " not matched"
    End If
    Exit Function
    
err_CompareUncWithDrivePath:
    MsgBox "Error no. " & CStr(Err.number) & ": " & Err.Description & vbNewLine & _
        "Was caused by " & Err.Source, vbOKOnly Or vbExclamation, "Error in function CompareUncWithDrivePath"
End Function
' ---------------------------------------------------------------------
'   References Windows Script Host Object Model (IWshRuntimeLibrary)
'   delivered in WINDOWS\system32\wshom.ocx
'   (This is not Windows Scripting Runtime)
'   Google 'vb.net compare drive path unc' to find similar articles
' ---------------------------------------------------------------------
Sub NetworkMapDrive()
    Dim strDrives                           As String
    Dim oWshNetwork                         As WshNetwork
    'Set WshNetwork = CreateObject(WScript.Network)
    Set oWshNetwork = New WshNetwork
    strDrives = "Network drive Mappings:" & Chr(13)
   
    Dim oDrives                             As New WshCollection
    '   The EnumNetworkDrives method returns a collection.
    '   This collection is an array that associates pairs of items  network drive local names and their associated UNC names.
    '   Even-numbered items in the collection represent local names of logical drives.
    '   Odd-numbered items represent the associated UNC share names.
    '   The first item in the collection is at index zero (0)
    Set oDrives = oWshNetwork.EnumNetworkDrives
    Dim i                                   As Integer
    For i = 0 To oDrives.Count - 1 Step 2
    strDrives = strDrives & "Drive " & oDrives.Item(i) & " = " & oDrives.Item(i + 1) & Chr(13)
    Next
    Set oWshNetwork = Nothing
    MsgBox strDrives
End Sub
' ---------------------------------------------------------------------

